/* 
 * Copyright (c) 2008 Texas Instruments and others.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      Texas Instruments - initial implementation
 *
 * */

package xdc.tools.closure;

/*!
 *  ======== xdc.tools.closure.Main ========
 *  Creates a directory containing all packages/files referenced by a
 *  configuration
 *
 *  The purpose of this tool is to build a single directory containing all
 *  packages loaded in the process of configuring an application. The bundle
 *  is built in a user-selected repository, which can be then relocated to
 *  a different location or to a different host, where the application
 *  being configured can be recompiled and relinked.
 *
 *  The closure tool can run only after {@link xdc.tools.configuro} runs
 *  and finishes the configuration step. Then, `xdc.tools.closure' detects
 *  all packages involved in the configuration and copies them to a new
 *  repository. All absolute paths in the relevant generated files that
 *  could be accessed by the user's makefile, when the application is
 *  rebuilt, are transformed into the paths relative to the the directory
 *  containing the closed bundle.
 *
 *  Some files in the copied packages are deleted to keep the size of
 *  the closed bundle manageable. The default filtering setting causes only
 *  selected files to be deleted, for example C sources or object files that
 *  are copied into a library. A user has an option of requesting other
 *  evels of filtering from not filtering anything to the most aggressive
 *  filtering of files, which leaves in the closed bundle only header files
 *  and libraries referenced by linker command files. Here are the filtering
 *  levels:
 *  @p(blist)
 *  - 0   - all files are preserved
 *  - 1   - C/C++ sources, object files, Java sources and class files,
 *          PDF files and internal RTSC files are deleted
 *  - 2   - left for future expansion, currently same as 1
 *  - 3   - libraries not referenced by the linker command file
 *          are also deleted
 *  - 4   - same as 3
 *  - 5   - only rov.xs file used by the ROV tools is left
 *  - 6   - all nonessential file are removed
 *  - 7+  - same as 6
 *  @p
 *
 *  @a(Examples)
 *  @p(code)
 *      xs xdc.tools.configuro -t gnu.targets.arm.GCArmv6 -o configPkg
 *      xs xdc.tools.closure -d D:/bundle C:/examples/memory/configPkg
 *  @p
 *
 *  After `xdc.tools.configuro` creates a configuration in `configPkg`,
 *  `xdc.tools.closure` gathers all packages loaded during the 
 *  configuration into a repository located in `D:/bundle`.
 */
metaonly module Main inherits xdc.tools.ICmd {

    override config String usage[] = [
        '[-v] [--filter=N] [-i pkg1 [-i pkg2 ...]] [-d dst_dir] package_base_dir'
    ];

instance:

    //!Print informative messages during execution
    @CommandOption('v')
    config Bool verboseFlag = false;

    /*!
     * ======== aggFilter ========
     * Alias for --filter=5
     *
     * This flag was used for requesting aggressive filtering. It was
     * superceded by 'filter'.
     */
    @CommandOption('f')
    config Bool aggFilter = false;

    /*!
     * ======== filter ========
     * Set the level of deleting nonessential files
     *
     * Library and header files are the only files that must be present for the
     * bundle generated by this tool to be used when building an application.
     * Other files are optional, and might be used by other tools if present.
     * The levels of filtering that can be specified by this parameter are:
     * 0   - all files are preserved
     * 1   - C sources, object files, Java sources and class files, PDF files 
     *       and internal RTSC files are deleted
     * 2   - left for future expansion, currently same as 1
     * 3   - libraries not referenced by the linker command file are also
     *       deleted
     * 4   - same as 3
     * 5   - only rov.xs file used by the ROV tools is left
     * 6   - all nonessential file are removed
     * 7+  - same as 6
     */
    @CommandOption('filter')
    config UInt filter = 3;

    //!Name of the destination directory
    @CommandOption('d')
    config String destination = "";

    //!Additional packages to be included in the bundle
    @CommandOption('i')
    config String includes[] = [];
}
/*
 *  @(#) xdc.tools.closure; 1, 0, 0,0; 4-14-2020 17:00:57; /db/ztree/library/trees/xdctools/xdctools-h04/src/
 */

